*  Character Table Generator 
*  "Char Dump," Version 1.0
*  By Adam Trionfo
*  August 26, 2009
*  July 12, 2010 - Fixed to Work on Real APF MP-100 Hardware
*  ---------------------------------------------------------
*
*  This program is written to be run on the circa 1978
*  APF MP-1000 computer game console which uses the
*  Motorola 6800 8-Bit CPU. 
*      
*  This program gives three choices to the user via the
*  standard APF menu:
*  
*     1. ALL CHARACTERS ($00-$FF)
*     2. ALPHANUMERIC CHARACTERS ($00-$7F)
*     3. SPECIAL SHAPE CHARACTERS ($80-$FF)
*     
*  After the user makes a choice, one of three screens is presented:
*
*     1. All Characters, $00-$FF are shown at the top-half of
*        the screen.  The bottom-half of the screen is blank.
*     2. All alphanumeric characters, $00-$7F, are shown on the
*        screen in a table format.  The table shows the user which
*        hex code will give the given character.  
*     3. All special shape characters, $80-$FF, are shown on the
*        screen in a table format.  The table shows the user which
*        hex code will give the given character.  
*
*  When the user makes a choice and the screen is filled with the
*  choosen characters, the characters are displayed indefinitely.
*  If the user wants to make another choice, then the console can
*  be reset as normal and another choice can be made. 

*  RAM Usage
    
CURSCR    EQU  $10   ; Current Screen Byte 
CURTAB    EQU  $12   ; Current Table Byte
LOWSCR    EQU  $200  ; Lowest Screen Byte
HISCR     EQU  $3FF  ; Highest Screen Byte
  
          
          ORG   $8000       ; Start of APF Cartridge Area
          
*  The BIOS uses the first five bytes of the cartridge  

          FCB   $BB         ; Tell BIOS a cart is present
          FDB   MENUSTR     ; Points to Menu string on cartridge 
          FCB   $33         ; LSB 4 bits is # of Menu Choices
*                             Menu has three choices          
          FCB   $00         ; Checksum-type byte.  Must be $00 else
*                             BIOS restarts right after Init

*  After a menu choice is made from the APF Menu via the
*  BIOS, then control passes to the program here ($8005).  

*  Check which choice was made at the menu
*  ---------------------------------------
          CMPB  #$31        ; Was button 1 pressed?
          BEQ   BUT1        ; If yes, then branch
          CMPB  #$32        ; Was button 2 pressed?
          BEQ   BUT2        ; If yes, then branch          
          CMPB  #$33        ; Was button 3 pressed?          
          BEQ   BUT3        ; If yes, then branch

*  User Choice 1 - ALL CHARACTERS ($00-$FF)
*  ----------------------------------------
*  All Characters, $00-$FF are shown at the top-half of
*  the screen.  The bottom-half of the screen is blank.
 
BUT1      LDX   #LOWSCR     ; Load Index Register with first
*                             byte of RAM in screen area
          LDA   #00         ; $00(@) will be first byte on screen 
          
FILLCHAR  STA   $00,X       ; Store current character on screen
          INCA              ; Increment current character
          INX               ; Increment current byte of screen
          CPX   #LOWSCR+$100 ; Has it reached middle of screen?  
          BHI   FILLCHAR    ; If not done, then loop until done
          
*  Fill bottom-half of screen
          LDX   #LOWSCR+$100 ; Middle of screen
          LDA   #$20         ; Load with dark-green space
FILLSPAC  STA   $00,X       ; Place $20 at current screen location
          INX               ; Increment current byte of screen
          CPX   #HISCR+1    ; Has it reached high-screen address?  
          BHI   FILLSPAC    ; If not done, then loop until done
         
          JMP   FOREVER     ; Jump to Endless Loop


*  User Choice 2 - ALPHANUMERIC CHARACTERS ($00-$7F)
*  -------------------------------------------------
*  All alphanumeric characters, $00-$7F, are shown on the
*  screen in a table format.  The table shows the user which
*  hex code will give the given character.  

*  Move Alphnumeric Table to screen ($200-$3FF)
*  These are characters $00-$7F
BUT2      
          LDX   #LOWSCR     ; Load Index Register with first
*                             byte of RAM in screen area
          STX   CURSCR      ; Store at Current Screen Byte
          LDX   #ALPHANUM   ; Load Index Register with first
*                             byte of Special Character Table 
          STX   CURTAB      ; Store at Current Table Byte
          
MOVALPHA  LDX   CURTAB      ; Load X with Current Table Byte                                  
          LDA   $00,X       ; Load A with Current Table Byte
          INX               ; Increment Current Table Byte
          STX   CURTAB      ; Store Current Table Byte
          LDX   CURSCR      ; Load X with Current Screen Byte
          STA   $00,X       ; Move Byte from Table to Screen
          INX               ; Increment Current Screen Byte
          STX   CURSCR      ; Store at Current Screen Byte
          CPX   #HISCR+1    ; Has the end of screen been reached?
          BHI   MOVALPHA    ; Move rest of table if not complete               
          JMP   FOREVER     ; Jump to Endless Loop

*  User Choice 3 - 3. SPECIAL SHAPE CHARACTERS ($80-$FF) 
*  -----------------------------------------------------
*  All special shape characters, $80-$FF, are shown on the
*  screen in a table format.  The table shows the user which
*  hex code will give the given character.  


*  Move Special Character Table to screen ($200-$3FF)
*  These are characters $80-$FF
BUT3                     
          LDX   #LOWSCR     ; Load Index Register with first
*                             byte of RAM in screen area
          STX   CURSCR      ; Store at Current Screen Byte
          LDX   #SPECIAL    ; Load Index Register with first
*                             byte of Special Character Table 
          STX   CURTAB      ; Store at Current Table Byte
          
MOVSPEC   LDX   CURTAB      ; Load X with Current Table Byte                                  
          LDA   $00,X       ; Load A with Current Table Byte
          INX               ; Increment Current Table Byte
          STX   CURTAB      ; Store Current Table Byte
          LDX   CURSCR      ; Load X with Current Screen Byte
          STA   $00,X       ; Move Byte from Table to Screen
          INX               ; Increment Current Screen Byte
          STX   CURSCR      ; Store at Current Screen Byte
          CPX   #HISCR+1    ; Has the end of screen been reached?
          BHI   MOVSPEC     ; Move rest of table if not complete               
          JMP   FOREVER     ; Jump to Endless Loop

*  All menu choices eventually branch here.  This loops
*  the program forever so that the characters continue to
*  be displayed.  
                
FOREVER  JMP   FOREVER      ; Show screen forever          
     

****************************************************************
*                                                              *
*  Data Area - There are NO program statements below this line *
*                                                              *
****************************************************************   


*  Alphanumeric Table
*  ------------------
*
*  This table holds the 128 alphanumeric characters in
*  an easy to use for the user manner.  The table itself
*  holds 512 bytes of pure data.

ALPHANUM
*  Row 00: $0200 - $021F
*  Top Row of Alphanumeric Table showing hex numbers $0-$F
          FCB   $60, $70, $60, $71, $60, $72, $60, $73
          FCB   $60, $74, $60, $75, $60, $76, $60, $77
          
          FCB   $60, $78, $60, $79, $60, $41, $60, $42
          FCB   $60, $43, $60, $44, $60, $45, $60, $46

*  Row 01: $0220 - $023F
*  Row $00 
          FCB   $70, $00, $60, $01, $60, $02, $60, $03
          FCB   $60, $04, $60, $05, $60, $06, $60, $07
          
          FCB   $60, $08, $60, $09, $60, $0A, $60, $0B
          FCB   $60, $0C, $60, $0D, $60, $0E, $60, $0F

*  Row 02: $0240 - $025F
*  Blank Line
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          FCB   $60, $60, $60, $60, $60, $60, $60, $60

*  Row 03: $0260 - $027F
*  Row $01
          FCB   $71, $10, $60, $11, $60, $12, $60, $13
          FCB   $60, $14, $60, $15, $60, $16, $60, $17
          
          FCB   $60, $18, $60, $19, $60, $1A, $60, $1B
          FCB   $60, $1C, $60, $1D, $60, $1E, $60, $1F


*  Row 04: $0280 - $029F
*  Blank Line
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          FCB   $60, $60, $60, $60, $60, $60, $60, $60

*  Row 05: $02A0 - $02BF
*  Row $02
          FCB   $72, $20, $60, $21, $60, $22, $60, $23
          FCB   $60, $24, $60, $25, $60, $26, $60, $27
          
          FCB   $60, $28, $60, $29, $60, $2A, $60, $2B
          FCB   $60, $2C, $60, $2D, $60, $2E, $60, $2F

*  Row 06: $$02C0 - $02DF
*  Blank Line
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          
          FCB   $60, $60, $60, $60, $60, $60, $60, $60
          FCB   $60, $60, $60, $60, $60, $60, $60, $60

*  Row 07: $02E0 - $02FF
*  Row $03
          FCB   $73, $30, $60, $31, $60, $32, $60, $33
          FCB   $60, $34, $60, $35, $60, $36, $60, $37
          
          FCB   $60, $38, $60, $39, $60, $3A, $60, $3B
          FCB   $60, $3C, $60, $3D, $60, $3E, $60, $3F

*  Row 08: $0300 - $031F
*  Blank Line
          FCB   $8C, $8C, $8C, $8C, $8C, $8C, $8C, $8C
          FCB   $8C, $8C, $8C, $8C, $8C, $8C, $8C, $8C
          
          FCB   $8C, $8C, $8C, $8C, $8C, $8C, $8C, $8C
          FCB   $8C, $8C, $8C, $8C, $8C, $8C, $8C, $8C

*  Row 09: $0320 - $033F
*  Row $04
          FCB   $34, $40, $20, $41, $20, $42, $20, $43
          FCB   $20, $44, $20, $45, $20, $46, $20, $47
          
          FCB   $20, $48, $20, $49, $20, $4A, $20, $4B
          FCB   $20, $4C, $20, $4D, $20, $4E, $20, $4F

*  Row 0A: $0340 - $035F
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 0B: $0360 - $037F
*  Row $05
          FCB   $35, $50, $20, $51, $20, $52, $20, $53
          FCB   $20, $54, $20, $55, $20, $56, $20, $57
          
          FCB   $20, $58, $20, $59, $20, $5A, $20, $5B
          FCB   $20, $5C, $20, $5D, $20, $5E, $20, $5F

*  Row 0C: $0380 - $039F
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 0D: $03A0 - $03BF
*  Row $06
          FCB   $36, $60, $20, $61, $20, $62, $20, $63
          FCB   $20, $64, $20, $65, $20, $66, $20, $67
          
          FCB   $20, $68, $20, $69, $20, $6A, $20, $6B
          FCB   $20, $6C, $20, $6D, $20, $6E, $20, $6F

*  Row 0E: $03C0 - $03DF
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 0F: $03E0 - $03FF
*  Row $07
          FCB   $37, $70, $20, $71, $20, $72, $20, $73
          FCB   $20, $74, $20, $75, $20, $76, $20, $77
          
          FCB   $20, $78, $20, $79, $20, $7A, $20, $7B
          FCB   $20, $7C, $20, $7D, $20, $7E, $20, $7F


*  Special Character Table
*  -----------------------
*
*  This table holds the 128 special characters in 
*  an easy to use for the user manner.  The table itself
*  holds 512 bytes of pure data.
       
SPECIAL
*  Row 00: $0200 - $021F
*  Top Row of Table showing hex numbers $0-$F
          FCB   $20, $30, $20, $31, $20, $32, $20, $33
          FCB   $20, $34, $20, $35, $20, $36, $20, $37
          
          FCB   $20, $38, $20, $39, $20, $01, $20, $02
          FCB   $20, $03, $20, $04, $20, $05, $20, $06
          
*  Row 01: $0220 - $023F
*  Row $80, Light Green Special Characters
          FCB   $38, $80, $20, $81, $20, $82, $20, $83
          FCB   $20, $84, $20, $85, $20, $86, $20, $87
          
          FCB   $20, $88, $20, $89, $20, $8A, $20, $8B
          FCB   $20, $8C, $20, $8D, $20, $8E, $20, $8F

*  Row 02: $0240 - $025F
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

* Row 03: $0260 - $027F
*  Row $90, Yellow Characters
          FCB   $39, $90, $20, $91, $20, $92, $20, $93
          FCB   $20, $94, $20, $95, $20, $96, $20, $97
          
          FCB   $20, $98, $20, $99, $20, $9A, $20, $9B
          FCB   $20, $9C, $20, $9D, $20, $9E, $20, $9F

*  Row 04: $0280 - $029F
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 05: $02A0 - $02BF
*  Row $A0, Dark-Blue Characters
          FCB   $01, $A0, $20, $A1, $20, $A2, $20, $A3
          FCB   $20, $A4, $20, $A5, $20, $A6, $20, $A7
          
          FCB   $20, $A8, $20, $A9, $20, $AA, $20, $AB
          FCB   $20, $AC, $20, $AD, $20, $AE, $20, $AF

*  Row 06: $$02C0 - $02DF
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 07: $02E0 - $02FF
*  Row $B0, Red Characters
          FCB   $02, $B0, $20, $B1, $20, $B2, $20, $B3
          FCB   $20, $B4, $20, $B5, $20, $B6, $20, $B7
          
          FCB   $20, $B8, $20, $B9, $20, $BA, $20, $BB
          FCB   $20, $BC, $20, $BD, $20, $BE, $20, $BF

*  Row 08: $0300 - $031F
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 09: $0320 - $033F
*  Row $C0, White Characters
          FCB   $03, $C0, $20, $C1, $20, $C2, $20, $C3
          FCB   $20, $C4, $20, $C5, $20, $C6, $20, $C7
          
          FCB   $20, $C8, $20, $C9, $20, $CA, $20, $CB
          FCB   $20, $CC, $20, $CD, $20, $CE, $20, $CF

*  Row 0A: $0340 - $035F
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 0B: $0360 - $037F
*  Row $D0, Light-Blue Characters
          FCB   $04, $D0, $20, $D1, $20, $D2, $20, $D3
          FCB   $20, $D4, $20, $D5, $20, $D6, $20, $D7
          
          FCB   $20, $D8, $20, $D9, $20, $DA, $20, $DB
          FCB   $20, $DC, $20, $DD, $20, $DE, $20, $DF

*  Row 0C: $0380 - $039F
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 0D: $03A0 - $03BF
*  Row $E0, Purple Characters
          FCB   $05, $E0, $20, $E1, $20, $E2, $20, $E3
          FCB   $20, $E4, $20, $E5, $20, $E6, $20, $E7
          
          FCB   $20, $E8, $20, $E9, $20, $EA, $20, $EB
          FCB   $20, $EC, $20, $ED, $20, $EE, $20, $EF

*  Row 0E: $03C0 - $03DF
*  Blank Line
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          
          FCB   $20, $20, $20, $20, $20, $20, $20, $20
          FCB   $20, $20, $20, $20, $20, $20, $20, $20

*  Row 0F: $03E0 - $03FF
*  Row $F0, Orange Characters
          FCB   $06, $F0, $20, $F1, $20, $F2, $20, $F3
          FCB   $20, $F4, $20, $F5, $20, $F6, $20, $F7
          
          FCB   $20, $F8, $20, $F9, $20, $FA, $20, $FB
          FCB   $20, $FC, $20, $FD, $20, $FE, $20, $FF


*  Cartridge Menu String
*  ---------------------
*  
*  The data required by the BIOS for the main menu is here.

MENUSTR   FCB   $E4         ; 4 spaces before Program Name string
          FCC   "CHARACTER TABLE GENERATOR"  ; Program Name String
          FCB   $E5         ; 5 spaces after text (two on next line)  
          FCB   $FE         ; Another 30 spaces
          FCC   "1. ALL CHARACTERS ($00-$FF)"
          FCB   $C5         ; Fill next 5 spaces with Fill Byte
          FCB   $60         ; Fill Byte (Blank Light green space) 
          FCC   "2. ALPHANUMERIC CHARACTERS"
          FCB   $C6         ; Fill next 6 spaces with Fill Byte
          FCB   $60         ; Fill Byte (Blank Light green space) 
          FCC   "   ($00-$7F)"
          FCB   $D4         ; Fill next 20 spaces with Fill Byte
          FCB   $60         ; Fill Byte (Blank Light green space)
          FCC   "3. SPECIAL SHAPE CHARACTERS"
          FCB   $C5         ; Fill next 5 spaces with Fill Byte
          FCB   $60         ; Fill Byte (Blank Light green space) 
          FCC   "   ($80-$FF)"
          FCB   $D4         ; Fill next 20 spaces with Fill Byte
          FCB   $60         ; Fill Byte (Blank Light green space)
          FCB   $DF         ; Fill next 31 spaces with Fill Byte
          FCB   $80         ; Fill Byte (Blank Black Space)
          FCB   $C1         ; Fill next space with Fill Byte
          FCB   $80         ; Fill Byte (Blank Black Space)
          FCB   $FF         ; Control Byte - End of String

CARTEND   FILL  $FF,$9000-CARTEND ; Pad with $FF for a 4K Cart

*  Notes of how the BIOS handles the cartridge menu strings
*  --------------------------------------------------------
*  How the control bytes work:
*  If high order bits are:
*     $E or $F, then fill with spaces ($20)
*     $C or $D, then fill with next byte that follows
*
*  How BIOS determine the number of spaces:
*     Control Byte is AND'ed with $1F, giving max result
*        
*  The maximum number of spaces varies:
*  
*      $FE =  30 spaces
*      $EF =  15 spaces
*      $DF =  31 spaces of fill byte 
*      $CF =  31 spaces of fill byte
*      
*  Keep in mind that $FF can't be used because it is 
*  the End of String identifier